#!/usr/bin/env python
#coding: utf-8

# polonizer.py - class for Polish language message parser
# Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.


from spdsimple.phonetizer import Latinizer
import re

class Polonizer(Latinizer):
	"""
	Specific-language (Polish) example.
	This class overrides one method
	for Ivona escaping characters.
	"""

	def __init__(self,charset='iso-8859-2'):
		Latinizer.__init__(self,
			charset=charset,
			charnames={
				' ':'spacja',
				'.':'kropka',
				'*':'gwiazdka',
				'+':'plus',
				'=':'równe',
				'#':'hasz',
				'€':'euro',
				'£':'funt',
				'&':'and',
				'@':'małpa',
				'~':'tylda',
				'°':'stopień',
				'_':'podkreślenie',
				'"':'cudzysłów',
				'/':'slesz',
				'\\':'bekslesz',
				'$':'dolar',
				'(':'nawias',
				')':'prawy nawias',
				'[':'nawias kwadratowy',
				']':'prawy nawias kwadratowy',
				'{':'klamra',
				'}':'prawa klamra',
				':':'dwukropek',
				';':'średnik',
				',':'przecinek',
				'!':'wykrzyknik',
				'?':'pytajnik',
				'-':'myślnik',
				'<':'mniejsze',
				'>':'większe'},
			keynames={
				'underscore':'podkreślenie',
				'space':'blank',
				' ':'blank',
				'esc':("esk~!ejp","eskejp"),
				'escape':("esk~!ejp","eskejp"),
				'double-quote':'cudzysłów'},
			transmap={
				'š':'sz',
				'Š':'Sz',
				'č':'cz',
				'Č':'cz',
				'ñ':'ńj',
				'Ñ':'Ńj',
				'å':'o',
				'Å':'O',
				'æ':'e',
				'Æ':'e',
				'ç':'s',
				'Ç':'s',
				'œ':'e',
				'Œ':'e',
				'ø':'e',
				'Ø':'e',
				'ä':'e',
				'Ä':'e',
				'ö':'e',
				'Ö':'e'
				},
			untrans="ąęśćńóźżłĄĘŚĆŃÓŹŻŁ",
			punctuation=",.?!\";:-",
			capformat="wielkie %s",
			charformat="znak %d",
			accents={'diaeresis':u'umlałt',
					'double acute':u'hungarumlałt'}
		)
	def key_repr(self,k):
		"""
		"""
		s=Latinizer.key_repr(self,k)
		if not isinstance(s,tuple):
			return s
		if not self.is_on('ivona_polish'):
			s=s[1]
		else:
			s=s[0]
		return s
	
	_speller={
		u'b':u'be',
		u'c':u'ce',
		u'd':u'de',
		u'f':u'ef',
		u'g':u'gie',
		u'h':u'ha',
		u'j':u'jot',
		u'k':u'ka',
		u'l':u'el',
		u'ł':u'eł',
		u'm':u'em',
		u'n':u'en',
		u'p':u'pe',
		u'q':u'ku',
		u'r':u'er',
		u's':u'es',
		u't':u'te',
		u'v':u'fał',
		u'w':u'wu',
		u'x':u'iks',
		u'y':u'igrek',
		u'z':u'zet',
		u'ż':u'żet'
	}

	_rstress=re.compile(r'^(.*)([aeiouy][^aeiouy]*)$',re.UNICODE)
	_pl_vowels=(u'a',u'ą',u'e',u'ę',u'i',u'o',u'ó',u'u',u'y')
	def _add_stress(self,s):
		r=self._rstress.match(s)
		if r:
			s=r.group(1)+'~!'+r.group(2)
		return s
	
	def pl_spell(self,s):
		s=list(s.strip().lower())
		last=s[-1]
		for n,a in enumerate(s):
			v=self._speller.get(a,a)
			if n > 0:
				t1=s[n-1][-1]
				t2=v[0]
				if ( t1 in self._pl_vowels and t2 in self._pl_vowels) or \
					(t1 == u's' and t2 == u'h') or \
					(t1 in (u'r',u's') and t2 == 'z'):
					v = "~'" + v
			if n == len(s)-1 and last != 'y':
				v=self._add_stress(v)
			s[n]=v
		return ''.join(s)
	
	
	_rlex1=re.compile(r'\b(etc|tty|sko|pko)\b',re.U | re.I | re.DOTALL)
	_rlex2=None #re.compile(r'\b(orca|capri)\b',re.U | re.I | re.DOTALL)
	_rldic={
		u'orca':u'orka',
		u'capri':u'kapri'
	}
	def lexicon(self,s):
		"""
		probably not very usable method demonstrates
		how lexicon method may be overriden in language module.
		In this case it simply spells words 'etc','tty','sko'
		and 'pko', and properly pronounces word 'orca' and 'capri'
		if Ivona synthesizer is used
		"""
		if not self.is_on('ivona_polish'):
			return None
		if not self.__class__._rlex2:
			self.__class__._rlex2=re.compile(
				u'\\b(%s)\\b' % ('|'.join(self.__class__._rldic.iterkeys())),
				re.DOTALL | re.UNICODE | re.I)
		s=self._rlex1.sub(lambda x:self.pl_spell(x.group(1)),s)
		return self.__class__._rlex2.sub(
			lambda x:self.__class__._rldic[x.group(1).lower()],
			s)
