#!/usr/bin/env python
#coding: utf-8

# spdsimple - class for creating Speech-Dispatcher modules
# Copyright (C) Bohdan R. Rau 2011-2012 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.

import sys,os,os.path,re,threading,pyspda,string,Queue,traceback

class udict(dict):
	def get(self,key,defval=None):
		if not key.startswith('force_'):
			v=dict.get(self,'force_'+key)
			if v is not None and v != 'no':
				return v
		return dict.get(self,key,defval)

_vmodes=["male1","male2","male3","female1","female2","female3","child_male","child_female"]

class SpeechModule(object):

	ressml=re.compile('<.*?>')

	@staticmethod
	def get_voice_id(vname):
		vname=vname.lower()
		try:
			return _vmodes.index(vname)+1
		except:
			return 0
		
	@staticmethod
	def ensure(a,mini,maxi):
		a=int(a)
		if a < mini: return mini
		if a > maxi: return maxi
		return a


	@staticmethod
	def exit(n=0):
		from ctypes import CDLL
		CDLL("libc.so.6").exit(0)



	def __init__(self,name,args):
		self.settings=udict({
			'sound_icons':'/usr/share/sounds/sound-icons',
			'audio_output_method':'pulse'})
		self.speaking=False
		self.initialized=False
		self.audio=False
		self.speak_sem=threading.Semaphore(0)
		self.bqueue=Queue.Queue()
		self.out_mutex=threading.Lock()
		self.log_mutex=threading.Lock()
		self.conf=None
		cpath=os.path.join(os.environ['HOME'],'.sd_'+name)
		if not os.path.exists(cpath):
			if len(args)> 1 and os.path.exists(args[1]):
				try:
					f=open(cpath,'w')
					f.write(open(args[1]).read())
					f.close()
					self.conf=cpath
				except:
					pass
		else:
			self.conf=cpath
		self.rtime=0
		self.voices={}		# map name -> voice
		self.voicelist=[]

	def strip_ssml(self,a):
		return self.ressml.sub('',a).\
			replace('&lt;','<').replace('&gt;','>').replace('&amp;','&')

	def cmd(self):
		a=sys.stdin.readline()
		if a == '':
			raise EOFError('End of data while reading command')
		a=a.upper().strip().split(None,1)
		return a

	def ignore_to_dot(self):
		while True:
			a=sys.stdin.readline()
			if a == '':
				raise EOFError('End of data while reading data')
			if a.strip()=='.':
				break
		self.reply(200,"OK IGNORED\n");

	def read_data(self):
		o=''
		while True:
			a=sys.stdin.readline()
			if a == '':
				raise EOFError('End of data while reading data')
			if a.strip()=='.':
				break
			if a.startswith('.'):
				a=a[1:]
			o+=a
		return o

	def read_settings(self,arg,current=False):
		vv=False
		vs=False
		va=False
		while True:
			a=sys.stdin.readline()
			if a == '':
				self.exit(0)
			if a.strip()=='.':
				break
			a=a.strip().split('=',1)
			if len(a) < 2:
				continue
			name=a[0].strip()
			value=a[1].strip()
			arg[name]=value
			if current:
				if name=='voice':
					vv=True
				if name=='language':
					va=True
				elif name=='synthesis_voice' and value != 'NULL':
					vs=True
		if current and (vv or va) and not vs:
			arg['synthesis_voice']=self.get_voice(vv,va)
		self.correct_settings()
		self.reply(200,"OK SETTINGS ACCEPTED");
		self.log('Settings got: %s\n' % str(arg))

	def reply(self,nr,lista):
		if isinstance(lista,str):
			lista=lista.split('\n')
		else:
			lista=lista[:]
		self.out_mutex.acquire()
		while len(lista) > 1:
			sys.stdout.write("%d-%s\n" % (int(nr),str(lista.pop(0))));
		sys.stdout.write("%d %s\n" % (int(nr),str(lista.pop(0))));
		sys.stdout.flush()
		self.out_mutex.release()

	def is_on(self,name,sets=None):
		if sets is None:
			sets=self.settings;
		val=sets.get(name,None)
		if not val:
			return False
		val=val.lower()
		return val == "on" or \
			val.startswith('t') or \
			val.startswith('y')

	def log(self,txt):
		if not self.is_on('enable_log'):
			return
		self.log_mutex.acquire()
		sys.stderr.write(txt)
		if not txt.endswith('\n'):
			sys.stderr.write('\n')
		self.log_mutex.release()



	def speak_fun(self):
		try:
			while True:
				self.speak_sem.acquire()
				self.reply(701,'BEGIN')
				if self.command == 'SOUNDICON':
					self.log("Icon [%s]" % self.spd)
					self.audio.playFile(os.path.join(self.settings['sound_icons'],self.spd))
				else:
					if self.command == 'SPEAK':
						self.spd=self.strip_ssml(self.spd)
					self.spd=self.spd.decode('utf-8')
					self.speaker()

				self.audio.finish()
				if self.audio.stopped:
					self.reply(703,'STOP')
				else:
					self.reply(702,'END')
				self.speaking=0
		except:
			self.bqueue.put(sys.exc_info())


	def list_voices(self):
		voices=self.get_voice_list()
		li=[]
		for a in voices:
			li.append('%s %s %s' % a)
		li.append('OK VOICE LIST SENT')
		self.reply(200,li)

	def run_th(self):
		try:
			while True:
				a=self.cmd()
				command=a[0]
				self.log("Command: %s\n" % (" ".join(a)))
				if command == 'QUIT':
					self.reply(200,"OK GOOD BYE")
					sys.exit(0)

				if command == 'INIT':
					if self.initialized:
						self.reply(401,'ALREADY INITIALIZED')
						continue
					self.read_config()
					if not self.init_module():
						raise Exception('Cannot configure module')
					self.initialized=True
					self.reply(299,"INITIALIZED OK")
					continue

				if not self.initialized:
					self.log("Client didn't call INIT!")
					self.read_config()
					if not self.init_module():
						raise Exception('Cannot configure module')
					self.initialized=True


				if command == 'AUDIO':
					self.reply(207,'OK RECEIVING AUDIO SETTINGS')
					self.read_settings(self.settings)
					self.init_audio()
					continue;

				if command == 'LOGLEVEL':
					self.reply(207,'OK RECEIVING LOGLEVEL')
					self.ignore_to_dot()
					continue;

				if command == 'SET':
					self.reply(207,'OK RECEIVING SETTINGS')
					self.read_settings(self.settings,True)
					continue;


				if command == 'LIST' and a[1] == 'VOICES':
					self.list_voices()
					continue

				if command in ('SPEAK','CHAR','KEY','SOUNDICON'):
					if self.speaking:
						self.reply(400,'BE PATIENT, BOY!')
						continue
					self.read_config()
					self.init_audio()
					self.reply(200,'OK GIMMA SPEECH DATA')
					self.spd=self.read_data().strip()
					self.log('DATA\n%s\nEND DATA' % self.spd)
					self.reply(200,'OK START SPEAKING')
					self.command=command
					self.speak()
					continue

				if command in ('STOP','PAUSE'):
					if self.audio:
						self.audio.stop()
					continue

				self.reply(500,"WHAT?")
		except SystemExit:
			self.bqueue.put('GOOD BYE')
		except:
			self.bqueue.put(sys.exc_info())

	def run(self):
		try:
			self.speak_thread=threading.Thread(target=self.speak_fun)
			self.speak_thread.start()
			self.input_thread=threading.Thread(target=self.run_th)
			self.input_thread.start()
			rs=self.bqueue.get()
			if isinstance(rs,str):
				sys.stderr.write("Exiting, %s\n" % rs);
				self.exit(0)
		except:
			rs=sys.exc_info()
		exc_type, exc_value, exc_traceback = rs
		traceback.print_exception(exc_type, exc_value, exc_traceback,
				limit=10, file=sys.stderr)
		self.exit(0)


	def read_config(self):

		if not self.conf:
			return
		try:
			s=os.stat(self.conf)
		except:
			self.conf=None
			return
		if self.rtime >=s.st_mtime:
			return
		self.rtime=s.st_mtime
		try:
			f=open(self.conf)
		except:
			self.conf=None
			return
		for x in self.settings.keys():
			if x.startswith('force_'):
				self.settings.pop(x)
		while True:
			a=f.readline()
			if a == '':
				break
			a=a.strip()
			if a.startswith('#'):
				continue
			a=a.split('=',1)
			if len(a) != 2:
				continue
			name=re.sub(r'[\s_]+','_',a[0].lower().strip())
			value=a[1].strip()
			self.settings[name] = value
		f.close()
		self.correct_settings()
		self.log("Module settings:\n%s" % str(self.settings))

	def speak(self):
		self.audio.enable()
		self.speaking=1
		self.speak_sem.release()


	def init_audio(self):
		if not self.audio:
			amode=pyspda.AUDIO_PULSE \
				if self.settings.get("audio_output_method","ao")=="pulse" \
				else pyspda.AUDIO_AO
			driver="default"
			param="default"
			if amode == pyspda.AUDIO_AO:
				driver=self.settings.get("audio_output_method","default");
				if driver == "nas":
					param=self.settings.get("audio_nas_server","default")
				elif driver == "oss":
					param=self.settings.get("audio_oss_device","default")
				elif driver == "alsa":
					param=self.settings.get("audio_alsa_device","default")
				else:
					driver="default"
			else:
				param=self.settings.get("audio_pulse_server","default")
			self.log("Audio mode requested '%s', real '%s'\n" % \
				(self.settings.get("audio_output_method",""),
				('pulse' if amode == pyspda.AUDIO_PULSE else 'ao')))
			self.log("Audio driver=%s, param=%s\n" % (driver,param));
			self.audio=pyspda.Spda(amode,driver=driver,param=param)

	def get_voice_list(self):
		vl=[]
		for a in self.voices:
			vl.append((a,self.voices[a].get('lang','en'),'None'))
		return vl

	def add_voice(self,name,voice):
		self.voices[name]=voice;
		self.voicelist.append((name,voice.get('lang','en'),voice.get('gender','m')))

	def get_current_voice(self):
		v=self.settings.get('synthesis_voice',self.voicelist[0][0])
		if not v in self.voices:
			v=self.self.voicelist[0][0]
		return self.voices[v]

	def get_voice(self,vv,va):
		lang=self.settings.get('language',None)
		vox=self.settings.get('voice','male1')
		try:
			vnum=int(vox[-1])
			gen=vox[0]
		except:
			vnum=1
			gen='m'
		voi=None
		for a in self.voicelist:
			if lang is not None and a[1] != lang:
				continue
			if not voi or a[2] == gen:
				voi=a
				if a[2] == gen:
					vnum -=1
					if vnum < 1:
						break
		if not voi:
			for a in self.voicelist:
				if not voi or a[2] == gen:
					voi=a
					if a[2] == gen:
						vnum -=1
						if vnum < 1:
							break
		if not voi:
			voi=self.voicelist[0]
		return voi[0]

	def init_module(self):
		"""
		should be overriden in child class
		return False to immediately exit module
		"""
		self.voices={'Default':{'gender':'m','lang':'en'}}
		self.voicelist=[('Default','en','m')]
		return True

	def speaker(self):
		"""
		virtual method
		speaks self.spd with command self.command
		"""
		pass
	
	def correct_settings(self):
		"""
		virtual method - called after settings change
		"""
		pass
	
