#!/usr/bin/env python
#coding: utf-8

# phonetizer.py - classes for multiple languages message parser
# Copyright (C) Bohdan R. Rau 2011 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.

import re,string

class Phonetizer(object):
	"""
	Universal (should be completely language independet)
	base class for message phonetization
	TODO: All internals MUST BE in Unicode!!!
	"""

	@staticmethod
	def unico(a):
		if isinstance(a,str):
			return a.decode('utf-8')
		return a

	@staticmethod
	def uni2cs(a,charset):
		if isinstance(a,unicode):
			return a.encode(charset,'ignore')
		return a

	def is_on(self,name):
		if self.settings is None:
			return False
		val=self.settings.get(name,None)
		if not val:
			return False
		val=val.lower()
		return val == "on" or \
			val.startswith('t') or \
			val.startswith('y')

	def __init__(self,
			charset='iso-8859-1', # charset used by synthesizer
			speakable=None,	# list of characters spoken as single character
			keynames=None,	# dictionary with keynames for 'KEY' command
			charnames=None, # dictionary with character names for 'CHAR'
			transmap=None,	# translation table for foreign letters
			capformat='capital %s', # format for 'capital'
			charformat='char %d',  # format for unspecified characters
			punctuation=",.?;:!-\"",
			):

		self.speakable=self.unico(speakable)
		self.charset=charset
		if keynames:
			for a in keynames.iterkeys():
				keynames[a]=self.unico(keynames[a])
		self.keynames=keynames
		self.transmap=transmap
		if punctuation.find(' ')<0:
			punctuation+=' '
		punctuation=self.unico(punctuation)
		if capformat.find('%s')<0:
			capformat += ' %s'
		self.capformat=self.unico(capformat)

		if charformat.find('%d')<0:
			charformat += ' %d'
		self.charformat=self.unico(charformat)
		self.chargen=None
		self.charnames={}

		if charnames:
			self.chargen={}
			for a in charnames:
				c=charnames[a]
				a=self.unico(a)[0]
				if not a in punctuation:
					self.chargen[ord(a)] = u' %s ' % self.unico(c)
				self.charnames[a]=self.unico(c)



	def split(self,message,command='SPEAK',settings=None,psome="()"):

		"""
		Returns message translated into list of parts.
		Each part may be string in specified encoding
		or tuple(msg,icon) where msg is string as above,
		icon is sound-icon name to play before speaking
		string.
		"""

		self.settings=settings
		self.punct_some=psome
		if command == 'KEY':
			return self.key(message)
		if command == 'CHAR':
			return self.char(message)
		if self.is_on('spelling_mode'):
			return list(self.char_repr(x) for x in message)
		return self.str_get(message)

	def key(self,s):
		"""
		Returns complex key name as single-element list.
		May be overriden f. ex. for languages using very specific
		words for complex key names.
		Input is string in UTF-8.
		"""
		if s == '_':
			s = ['underscore']
		else:
			s=s.split('_')
		s=list(self.key_repr(x) for x in s)
		return [' '.join(s)]

	def key_repr(self,k):
		"""
		Returns single key name.
		Probably not needs to be overriden
		Input is string in UTF-8
		Output is string in charset
		"""

		if self.keynames and k in self.keynames:
			return self.uni2cs(self.keynames[k],self.charset)
		if len(k) == 1:
			return self.char_repr(k,nocap=True)
		return self.uni2cs(self.unico(k),self.charset)

	def char(self,s):
		"""
		Returns single character name as single
		element list. Do not override this
		method, override char_repr below if you need!
		Input is string in UTF-8
		"""

		if not s:
			return []
		if s == '#':
			return [self.char_repr(u'#')]
		if s.startswith('#x'):
			try:
				s=unichr(int(s[2:],16))
			except:
				s='?'
		elif s.startswith('#'):
			try:
				s=unichr(int(s[1:],10))
			except:
				s=u'?'
		return [self.char_repr(s[0])]

	def cap(self,s):
		"""
		capitalizer
		"""
		mode=self.settings.get('cap_let_recogn','none')
		if mode == 'spell':
			s=self.capformat % s
		s=self.uni2cs(s,self.charset)
		if mode == 'icon':
			return (s,'capital')
		return s

	def char_repr(self,c,nocap=False):
		"""
		Returns single character name as string
		in specified encoding.
		Input is any UTF-8 or Unicode
		"""
		c=self.unico(c)
		if len(c) < 1:
			return ''
		c=c[0]
		if c in self.charnames:
			s=self.charnames[c]
			if not nocap and c.isupper():
				return self.cap(s)
			return self.uni2cs(s,self.charset)
		if self.speakable and c in self.speakable:
			if not nocap and c.isupper():
				return self.cap(c.lower())
			return self.uni2cs(c,self.charset)
		return self.uni2cs(self.charformat % ord(c),self.charset)

	def str_get(self,s):
		"""
		Returns message as list of sentences
		or (if spelling mode is set) as
		list of character names. Do not override
		this function, override str_repr below!
		"""

		if not self.is_on('spelling_mode'):
			if len(s.strip()) == 0:
				if not self.is_on('announce_blank'):
					return None
			return self.str_repr(s)
		os=[]
		for a in s:
			os.append(self.char_repr(a))
		return os

	def str_repr(self,s):

		"""
		Very simple method for translation
		strings into sentences for synthesizer.
		You probably will override this method,
		but it is still usable for first tests
		of new synthesizers/modules.
		Especially translating with 'chargen'
		is not very good idea...
		"""

		if len(s) == 1:
			return [self.char_repr(s)]
		if self.transmap:
			s=s.translate(self.transmap)
		if self.chargen:
			s=s.translate(self.chargen)
		return [self.str_convert(s)]

	def str_convert(self,s):
		"""

		"""
		if len(s)==1:
			return self.char_repr(s)
		return s.encode(self.charset,'replace')
	
	


class Latinizer(Phonetizer):
	"""
	Base class for Latin script languages.
	For Cirillic based languages (or for any language
	using alphabetic notation and punctuation characters
	as in Western languages) only builtin translation array
	must be extended.
	"""

	def __init__(self,
			charset='iso-8859-1', # as above
			speakable=None,		# as above
			keynames=None,		# as above
			charnames=None,		# as above
			transmap=None,		# extra characters for translation
								# builtin table contains all letter
								# from range 0xc0 - 0x180
			untrans=None,		# not transladed characters
			capformat="capital",	# as above
			charformat="char %d",	# as above
			stopchars=".?!",			# phrase end marker
			punctuation=",.?!-;:\"",
			accents=None			# accent names for accented letters

			):
		self.stopchars=stopchars
		self.stopre=re.compile(u"^(.*?[%s])\s+(.*)$" %
				stopchars.replace('[',r'\[').replace(']','\]').decode('utf-8'),
			re.DOTALL)
		tmap={
			192:u'A',193:u'A',194:u'A',195:u'A',196:u'E',197:u'O',198:u'E',
			199:u'C',200:u'E',201:u'E',202:u'E',203:u'E',204:u'I',205:u'I',
			206:u'I',207:u'I',208:u'D',209:u'N',210:u'O',211:u'O',212:u'O',
			213:u'O',214:u'E',216:u'E',217:u'U',218:u'U',219:u'U',220:u'I',
			221:u'Y',222:u'Þ',223:u's',224:u'a',225:u'a',226:u'a',227:u'a',
			228:u'e',229:u'o',230:u'e',231:u's',232:u'e',233:u'e',234:u'e',
			235:u'e',236:u'i',237:u'i',238:u'i',239:u'i',240:u'd',241:u'n',
			242:u'o',243:u'o',244:u'o',245:u'o',246:u'e',248:u'e',249:u'u',
			250:u'u',251:u'u',252:u'i',253:u'y',254:u't',255:u'y',256:u'A',
			257:u'a',258:u'A',259:u'a',260:u'On',261:u'on',262:u'C',263:u'c',
			264:u'C',265:u'c',266:u'C',267:u'c',268:u'C',269:u'c',270:u'D',
			271:u'd',272:u'D',273:u'd',274:u'E',275:u'e',276:u'E',277:u'e',
			278:u'E',279:u'e',280:u'En',281:u'en',282:u'E',283:u'e',284:u'G',
			285:u'g',286:u'G',287:u'g',288:u'G',289:u'g',290:u'G',291:u'g',
			292:u'H',293:u'h',294:u'H',295:u'h',296:u'I',297:u'i',298:u'I',
			299:u'i',300:u'L',301:u'l',302:u'I',303:u'i',304:u'I',305:u'i',
			306:u'IJ',307:u'ij',308:u'J',309:u'j',310:u'K',311:u'k',312:u'k',
			313:u'L',314:u'l',315:u'L',316:u'l',317:u'L',318:u'l',319:u'L',
			320:u'l',321:u'L',322:u'l',323:u'N',324:u'n',325:u'N',326:u'n',
			327:u'N',328:u'n',329:u'N',330:u'n',331:u'ng',332:u'O',333:u'o',
			334:u'O',335:u'o',336:u'O',337:u'o',338:u'E',339:u'e',340:u'R',
			341:u'r',342:u'R',343:u'r',344:u'R',345:u'r',346:u'S',347:u's',
			348:u'S',349:u's',350:u'S',351:u's',352:u'S',353:u's',354:u'T',
			355:u't',356:u'T',357:u't',358:u'T',359:u't',360:u'U',361:u'u',
			362:u'U',363:u'u',364:u'U',365:u'u',366:u'U',367:u'u',368:u'U',
			369:u'u',370:u'U',371:u'u',372:u'W',373:u'w',374:u'Y',375:u'y',
			376:u'y',377:u'Z',378:u'z',379:u'Z',380:u'z',381:u'Z',382:u'z'}

		if transmap:
			for a in transmap:
				tmap[ord(self.unico(a))] = self.unico(transmap[a])
		if untrans:
			untrans=self.unico(untrans)
			for a in untrans:
				a=ord(a)
				if a in tmap:
					tmap.pop(a)
		s=string.ascii_letters + string.digits
		if speakable:
			s += speakable
		speakable=set(c for c in s.decode('utf-8'))

		knames=None
		if keynames:
			knames={}
			for a in keynames:
				knames[self.unico(a)]=self.uni2cs(keynames[a],charset)

		self.unspacer=re.compile(r'\s+',re.DOTALL)

		self.letternames={
			u'ß':{'name':u'sharp es','type':'let','accent':None},
			u'À':{'name':u'a','type':'let','accent':'grave'},
			u'Á':{'name':u'a','type':'let','accent':'acute'},
			u'Â':{'name':u'a','type':'let','accent':'circumflex'},
			u'Ã':{'name':u'a','type':'let','accent':'tilde'},
			u'Ä':{'name':u'a','type':'let','accent':'diaeresis'},
			u'Å':{'name':u'a','type':'let','accent':'ring above'},
			u'Æ':{'name':u'ae','type':'lig','accent':None},
			u'Ç':{'name':u'c','type':'let','accent':'cedilla'},
			u'È':{'name':u'e','type':'let','accent':'grave'},
			u'É':{'name':u'e','type':'let','accent':'acute'},
			u'Ê':{'name':u'e','type':'let','accent':'circumflex'},
			u'Ë':{'name':u'e','type':'let','accent':'diaeresis'},
			u'Ì':{'name':u'i','type':'let','accent':'grave'},
			u'Í':{'name':u'i','type':'let','accent':'acute'},
			u'Î':{'name':u'i','type':'let','accent':'circumflex'},
			u'Ï':{'name':u'i','type':'let','accent':'diaeresis'},
			u'Ð':{'name':u'eth','type':'let','accent':None},
			u'Ñ':{'name':u'n','type':'let','accent':'tilde'},
			u'Ò':{'name':u'o','type':'let','accent':'grave'},
			u'Ó':{'name':u'o','type':'let','accent':'acute'},
			u'Ô':{'name':u'o','type':'let','accent':'circumflex'},
			u'Õ':{'name':u'o','type':'let','accent':'tilde'},
			u'Ö':{'name':u'o','type':'let','accent':'diaeresis'},
			u'Ø':{'name':u'o','type':'let','accent':'stroke'},
			u'Ù':{'name':u'u','type':'let','accent':'grave'},
			u'Ú':{'name':u'u','type':'let','accent':'acute'},
			u'Û':{'name':u'u','type':'let','accent':'circumflex'},
			u'Ü':{'name':u'u','type':'let','accent':'diaeresis'},
			u'Ý':{'name':u'y','type':'let','accent':'acute'},
			u'Þ':{'name':u'thorn','type':'let','accent':None},
			u'à':{'name':u'a','type':'let','accent':'grave'},
			u'á':{'name':u'a','type':'let','accent':'acute'},
			u'â':{'name':u'a','type':'let','accent':'circumflex'},
			u'ã':{'name':u'a','type':'let','accent':'tilde'},
			u'ä':{'name':u'a','type':'let','accent':'diaeresis'},
			u'å':{'name':u'a','type':'let','accent':'ring above'},
			u'æ':{'name':u'ae','type':'lig','accent':None},
			u'ç':{'name':u'c','type':'let','accent':'cedilla'},
			u'è':{'name':u'e','type':'let','accent':'grave'},
			u'é':{'name':u'e','type':'let','accent':'acute'},
			u'ê':{'name':u'e','type':'let','accent':'circumflex'},
			u'ë':{'name':u'e','type':'let','accent':'diaeresis'},
			u'ì':{'name':u'i','type':'let','accent':'grave'},
			u'í':{'name':u'i','type':'let','accent':'acute'},
			u'î':{'name':u'i','type':'let','accent':'circumflex'},
			u'ï':{'name':u'i','type':'let','accent':'diaeresis'},
			u'ð':{'name':u'eth','type':'let','accent':None},
			u'ñ':{'name':u'n','type':'let','accent':'tilde'},
			u'ò':{'name':u'o','type':'let','accent':'grave'},
			u'ó':{'name':u'o','type':'let','accent':'acute'},
			u'ô':{'name':u'o','type':'let','accent':'circumflex'},
			u'õ':{'name':u'o','type':'let','accent':'tilde'},
			u'ö':{'name':u'o','type':'let','accent':'diaeresis'},
			u'ø':{'name':u'o','type':'let','accent':'stroke'},
			u'ù':{'name':u'u','type':'let','accent':'grave'},
			u'ú':{'name':u'u','type':'let','accent':'acute'},
			u'û':{'name':u'u','type':'let','accent':'circumflex'},
			u'ü':{'name':u'u','type':'let','accent':'diaeresis'},
			u'ý':{'name':u'y','type':'let','accent':'acute'},
			u'þ':{'name':u'thorn','type':'let','accent':None},
			u'ÿ':{'name':u'y','type':'let','accent':'diaeresis'},
			u'Ā':{'name':u'a','type':'let','accent':'macron'},
			u'ā':{'name':u'a','type':'let','accent':'macron'},
			u'Ă':{'name':u'a','type':'let','accent':'breve'},
			u'ă':{'name':u'a','type':'let','accent':'breve'},
			u'Ą':{'name':u'a','type':'let','accent':'ogonek'},
			u'ą':{'name':u'a','type':'let','accent':'ogonek'},
			u'Ć':{'name':u'c','type':'let','accent':'acute'},
			u'ć':{'name':u'c','type':'let','accent':'acute'},
			u'Ĉ':{'name':u'c','type':'let','accent':'circumflex'},
			u'ĉ':{'name':u'c','type':'let','accent':'circumflex'},
			u'Ċ':{'name':u'c','type':'let','accent':'dot above'},
			u'ċ':{'name':u'c','type':'let','accent':'dot above'},
			u'Č':{'name':u'c','type':'let','accent':'caron'},
			u'č':{'name':u'c','type':'let','accent':'caron'},
			u'Ď':{'name':u'd','type':'let','accent':'caron'},
			u'ď':{'name':u'd','type':'let','accent':'caron'},
			u'Đ':{'name':u'd','type':'let','accent':'stroke'},
			u'đ':{'name':u'd','type':'let','accent':'stroke'},
			u'Ē':{'name':u'e','type':'let','accent':'macron'},
			u'ē':{'name':u'e','type':'let','accent':'macron'},
			u'Ĕ':{'name':u'e','type':'let','accent':'breve'},
			u'ĕ':{'name':u'e','type':'let','accent':'breve'},
			u'Ė':{'name':u'e','type':'let','accent':'dot above'},
			u'ė':{'name':u'e','type':'let','accent':'dot above'},
			u'Ę':{'name':u'e','type':'let','accent':'ogonek'},
			u'ę':{'name':u'e','type':'let','accent':'ogonek'},
			u'Ě':{'name':u'e','type':'let','accent':'caron'},
			u'ě':{'name':u'e','type':'let','accent':'caron'},
			u'Ĝ':{'name':u'g','type':'let','accent':'circumflex'},
			u'ĝ':{'name':u'g','type':'let','accent':'circumflex'},
			u'Ğ':{'name':u'g','type':'let','accent':'breve'},
			u'ğ':{'name':u'g','type':'let','accent':'breve'},
			u'Ġ':{'name':u'g','type':'let','accent':'dot above'},
			u'ġ':{'name':u'g','type':'let','accent':'dot above'},
			u'Ģ':{'name':u'g','type':'let','accent':'cedilla'},
			u'ģ':{'name':u'g','type':'let','accent':'cedilla'},
			u'Ĥ':{'name':u'h','type':'let','accent':'circumflex'},
			u'ĥ':{'name':u'h','type':'let','accent':'circumflex'},
			u'Ħ':{'name':u'h','type':'let','accent':'stroke'},
			u'ħ':{'name':u'h','type':'let','accent':'stroke'},
			u'Ĩ':{'name':u'i','type':'let','accent':'tilde'},
			u'ĩ':{'name':u'i','type':'let','accent':'tilde'},
			u'Ī':{'name':u'i','type':'let','accent':'macron'},
			u'ī':{'name':u'i','type':'let','accent':'macron'},
			u'Ĭ':{'name':u'i','type':'let','accent':'breve'},
			u'ĭ':{'name':u'i','type':'let','accent':'breve'},
			u'Į':{'name':u'i','type':'let','accent':'ogonek'},
			u'į':{'name':u'i','type':'let','accent':'ogonek'},
			u'İ':{'name':u'i','type':'let','accent':'dot above'},
			u'Ĵ':{'name':u'j','type':'let','accent':'circumflex'},
			u'ĵ':{'name':u'j','type':'let','accent':'circumflex'},
			u'Ķ':{'name':u'k','type':'let','accent':'cedilla'},
			u'ķ':{'name':u'k','type':'let','accent':'cedilla'},
			u'ĸ':{'name':u'kra','type':'let','accent':None},
			u'Ĺ':{'name':u'l','type':'let','accent':'acute'},
			u'ĺ':{'name':u'l','type':'let','accent':'acute'},
			u'Ļ':{'name':u'l','type':'let','accent':'cedilla'},
			u'ļ':{'name':u'l','type':'let','accent':'cedilla'},
			u'Ľ':{'name':u'l','type':'let','accent':'caron'},
			u'ľ':{'name':u'l','type':'let','accent':'caron'},
			u'Ŀ':{'name':u'l','type':'let','accent':'middle dot'},
			u'ŀ':{'name':u'l','type':'let','accent':'middle dot'},
			u'Ł':{'name':u'l','type':'let','accent':'stroke'},
			u'ł':{'name':u'l','type':'let','accent':'stroke'},
			u'Ń':{'name':u'n','type':'let','accent':'acute'},
			u'ń':{'name':u'n','type':'let','accent':'acute'},
			u'Ņ':{'name':u'n','type':'let','accent':'cedilla'},
			u'ņ':{'name':u'n','type':'let','accent':'cedilla'},
			u'Ň':{'name':u'n','type':'let','accent':'caron'},
			u'ň':{'name':u'n','type':'let','accent':'caron'},
			u'Ŋ':{'name':u'eng','type':'let','accent':None},
			u'ŋ':{'name':u'eng','type':'let','accent':None},
			u'Ō':{'name':u'o','type':'let','accent':'macron'},
			u'ō':{'name':u'o','type':'let','accent':'macron'},
			u'Ŏ':{'name':u'o','type':'let','accent':'breve'},
			u'ŏ':{'name':u'o','type':'let','accent':'breve'},
			u'Ő':{'name':u'o','type':'let','accent':'double acute'},
			u'ő':{'name':u'o','type':'let','accent':'double acute'},
			u'Ŕ':{'name':u'r','type':'let','accent':'acute'},
			u'ŕ':{'name':u'r','type':'let','accent':'acute'},
			u'Ŗ':{'name':u'r','type':'let','accent':'cedilla'},
			u'ŗ':{'name':u'r','type':'let','accent':'cedilla'},
			u'Ř':{'name':u'r','type':'let','accent':'caron'},
			u'ř':{'name':u'r','type':'let','accent':'caron'},
			u'Ś':{'name':u's','type':'let','accent':'acute'},
			u'ś':{'name':u's','type':'let','accent':'acute'},
			u'Ŝ':{'name':u's','type':'let','accent':'circumflex'},
			u'ŝ':{'name':u's','type':'let','accent':'circumflex'},
			u'Ş':{'name':u's','type':'let','accent':'cedilla'},
			u'ş':{'name':u's','type':'let','accent':'cedilla'},
			u'Š':{'name':u's','type':'let','accent':'caron'},
			u'š':{'name':u's','type':'let','accent':'caron'},
			u'Ţ':{'name':u't','type':'let','accent':'cedilla'},
			u'ţ':{'name':u't','type':'let','accent':'cedilla'},
			u'Ť':{'name':u't','type':'let','accent':'caron'},
			u'ť':{'name':u't','type':'let','accent':'caron'},
			u'Ŧ':{'name':u't','type':'let','accent':'stroke'},
			u'ŧ':{'name':u't','type':'let','accent':'stroke'},
			u'Ũ':{'name':u'u','type':'let','accent':'tilde'},
			u'ũ':{'name':u'u','type':'let','accent':'tilde'},
			u'Ū':{'name':u'u','type':'let','accent':'macron'},
			u'ū':{'name':u'u','type':'let','accent':'macron'},
			u'Ŭ':{'name':u'u','type':'let','accent':'breve'},
			u'ŭ':{'name':u'u','type':'let','accent':'breve'},
			u'Ů':{'name':u'u','type':'let','accent':'ring above'},
			u'ů':{'name':u'u','type':'let','accent':'ring above'},
			u'Ű':{'name':u'u','type':'let','accent':'double acute'},
			u'ű':{'name':u'u','type':'let','accent':'double acute'},
			u'Ų':{'name':u'u','type':'let','accent':'ogonek'},
			u'ų':{'name':u'u','type':'let','accent':'ogonek'},
			u'Ŵ':{'name':u'w','type':'let','accent':'circumflex'},
			u'ŵ':{'name':u'w','type':'let','accent':'circumflex'},
			u'Ŷ':{'name':u'y','type':'let','accent':'circumflex'},
			u'ŷ':{'name':u'y','type':'let','accent':'circumflex'},
			u'Ÿ':{'name':u'y','type':'let','accent':'diaeresis'},
			u'Ź':{'name':u'z','type':'let','accent':'acute'},
			u'ź':{'name':u'z','type':'let','accent':'acute'},
			u'Ż':{'name':u'z','type':'let','accent':'dot above'},
			u'ż':{'name':u'z','type':'let','accent':'dot above'},
			u'Ž':{'name':u'z','type':'let','accent':'caron'},
			u'ž':{'name':u'z','type':'let','accent':'caron'},
			u'ŉ':{'name':u'n','type':'let','accent':'apostrophe'},
			u'ſ':{'name':u's','type':'let','accent':'long'},


		}
		self.accents=accents
		Phonetizer.__init__(self,
			charset=charset,
			speakable=speakable,
			keynames=knames,
			charnames=charnames,
			transmap=tmap,
			capformat=capformat,
			charformat=charformat)

	def acc(self,ac):
		if self.accents is not None:
			return self.unico(self.accents.get(ac,ac))
		return self.unico(ac)

	def char_repr(self,c,nocap=False):
		if len(c) == 1 and  not \
				(self.speakable and c in self.speakable):
			z=self.letternames.get(c,None)
			if z:
				if z['type'] == 'lig':
					s="%s %s %s" %(self.acc('ligature'),z['name'][0],z['name'][1])
				else:
					s=z['name']
				if z['accent'] is not None:
					s+=u' '+self.acc(z['accent'])
				if not nocap and c.isupper():
					return self.cap(s)
				return self.uni2cs(s,self.charset)
		return Phonetizer.char_repr(self,c,nocap)

	def str_repr(self,s):
		if len(s) == 1:
			return [self.char_repr(s)]
		if self.chargen:
			s=s.translate(self.chargen)
		s=self.unspacer.sub(' ',self.lexicon(s) or s)
		sq=[]
		while True:
			r=self.stopre.search(s)
			if r:
				sq.append(r.group(1))
				s=r.group(2)
			else:
				sq.append(s)
				break
		return list(self.str_convert(s) for s in sq)

	def str_convert(self,s):
		if len(s) == 1:
			return self.char_repr(s)
		if self.transmap:
			s=s.translate(self.transmap)
		return s.encode(self.charset,'replace')

	def lexicon(self,s):
		"""
		virtual method
		"""
		return None
